@@ -4,18 +4,10 @@ use warnings;
 use Moo;
 use Log::Any qw/$log/;
 use Carp qw/confess/;
-use SQL::DB::Expr qw/_quote/;
+use SQL::DB::Expr qw/_bval/;
 use Sub::Install qw/install_sub/;
-use Sub::Exporter -setup => {
-    exports => ['load_schema'],
-    groups  => {
-        all     => ['load_schema'],
-        default => [],
-    },
-};
 
-our $VERSION = '0.19_11';
-my %schema;
+our $VERSION = '0.19_13';
 
 # Ordinals for DBI->column_info() results
 use constant {
@@ -42,15 +34,17 @@ use constant {
 # Object definition
 
 has 'name' => (
-    is       => 'rw',
+    is       => 'ro',
     required => 1,
 );
 
-has '_package_root' => (
-    is       => 'rw',
-    init_arg => undef,
+has 'package_root' => (
+    is       => 'ro',
+    required => 1,
 );
 
+has 'load' => ( is => 'ro', );
+
 has '_tables' => (
     is       => 'ro',
     init_arg => undef,
@@ -59,19 +53,34 @@ has '_tables' => (
 
 sub _getglob { no strict 'refs'; \*{ $_[0] } }
 
+around BUILDARGS => sub {
+    my $orig  = shift;
+    my $class = shift;
+    my %args  = @_;
+
+    ( $args{package_root} = $args{name} ) =~ tr/a-zA-Z0-9/_/cs;
+    $args{package_root} = __PACKAGE__ . '::' . $args{package_root};
+
+    return $class->$orig(%args);
+};
+
 sub BUILD {
     my $self = shift;
-    ( my $clean = $self->name ) =~ tr/a-zA-Z0-9/_/cs;
-    $self->_package_root( __PACKAGE__ . '::' . $clean );
-    $schema{ $self->name } = $self;
-    $log->debug( "Schema " . $self->name . " created" );
+
+    if ( $self->load ) {
+        require Module::Load;
+        Module::Load::load( $self->name );
+        $self->define( $self->name->definition );
+        $self->name->clear;
+        $log->debug( 'Loaded schema', $self->name );
+    }
 }
 
 sub define {
     my $self = shift;
     my $data = shift;
 
-    my $package_root = $self->_package_root;
+    my $package_root = $self->package_root;
     my $tables       = $self->_tables;
 
     foreach my $colref (@$data) {
@@ -111,7 +120,7 @@ sub define {
         }
         $tables->{$table}++;
 
-        my $col = $colref->[COLUMN_NAME];
+        my $col = lc $colref->[COLUMN_NAME];
 
         if ( $col eq 'new' ) {
             confess "Column name 'new' (table/view '$table') clashes with "
@@ -126,8 +135,8 @@ sub define {
                 code => sub {
                     my $table_expr = shift;
                     SQL::DB::Expr->new(
-                        _txt   => [ $table_expr->_alias . '.' . $col ],
-                        _btype => $type,
+                        _txt  => [ $table_expr->_alias . '.' . $col ],
+                        _type => $type,
                     );
                 },
                 into => $srow,
@@ -143,15 +152,14 @@ sub define {
                     if (@_) {
                         my $val = shift;
                         return SQL::DB::Expr->new(
-                            _txt     => [ $col . ' = ', _quote( $val, $type ) ],
-                            _btype   => $type,
-                            _bvalues => [$val],
+                            _txt  => [ $col . ' = ', _bval( $val, $type ) ],
+                            _type => $type,
                         );
                     }
 
                     return SQL::DB::Expr->new(
-                        _txt   => [$col],
-                        _btype => $type,
+                        _txt  => [$col],
+                        _type => $type,
                     );
                 },
                 into => $urow,
@@ -191,7 +199,7 @@ sub srow {
         if ( !exists $self->_tables->{$name} ) {
             confess "Table not defined in schema: $name";
         }
-        my $class = $self->_package_root . '::Srow::' . $name;
+        my $class = $self->package_root . '::Srow::' . $name;
         my $srow = $class->new( _txt => [$name], _alias => $name );
         return $srow unless (wantarray);
         push( @ret, $srow );
@@ -207,7 +215,7 @@ sub urow {
         if ( !exists $self->_tables->{$name} ) {
             confess "Table not defined in schema: $name";
         }
-        my $class = $self->_package_root . '::Urow::' . $name;
+        my $class = $self->package_root . '::Urow::' . $name;
         my $urow = $class->new( _txt => [$name] );
         return $urow unless (wantarray);
         push( @ret, $urow );
@@ -215,21 +223,6 @@ sub urow {
     return @ret;
 }
 
-# Class functions
-
-sub load_schema {
-    my $name = shift;
-    eval "require $name;";
-    if ($@) {
-        confess $@;
-    }
-    elsif ( !exists $schema{$name} ) {
-        confess "$name did not load properly";
-    }
-    $log->debug("load_schema($name) succeeded");
-    return $schema{$name};
-}
-
 1;
 
 # vim: set tabstop=4 expandtab:
